/* CUSTOM.C - Test harness for the reporterror diagnostic module.

            custom.c provides the interface between testharness
            functions and the C functions being tested. It contains
            two functions:
            
            - show_help(), which provides user information about
                           the tests to be run, such as script names
                           and any external requirements, apart from
                           test scripts, needed to do the tests.
                           
            - calladapter(), which allows individual function tests to be
                           run by mapping the function name and arguments
                           in the test definition to a function call,
                           executing the function, and writing test results
                           to stdout after the test has been run.

            Tests are driven by testharness.c, which uses command line
            arguments and options to set run conditions and specify
            which test scripts will be run. Testharness interprets each
            test script, using the calladapter() function in this module
            to exercise the functions being tested, and writes the results
            of each call to stdout.

            =======
            N O T E
            =======
            The four functions in custom.c whose names staer with 'custom'
            are only there so that the this source file can be compiled
            and run as issued. They should be removed as part of adapting
            this module to test your own functions.

            Written by M.C.Gregorie.

            Version history
               
            1.0   26/05/20   Initial release
            2.0   31/07/20   renamed testharness() to runtest()
*/

#define VERSION         "2.0"
#define COPYRIGHT       "(c) M.C.Gregorie, 2020"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "runtest.h"
#include "custom.h"

/*
   Private function prototypes
*/
int  example(char *s, int n);
char examplechr(char c);
char *examplestr(char *s);
int  exampletf(int bool);
void testargs(char s[MAXARGS][MAXLINE], int n[MAXARGS]);
void testend(int reply, char *fn, char s[MAXARGS][MAXLINE], int n[MAXARGS]);
void teststart(char *fn, char s[MAXARGS][MAXLINE], int n[MAXARGS]);

/*
   A code-specific call adapter which must be modified to call the
   set of functions to be tested.

   Entry and exit is reported if debug level is 2 or more.
   This example implementation contains the code to run functions
   whose names all start with 'example'. Their prototypes and
   definitions only exist to allow custom.c, as released, to be
   compiled and run. data/custom01.txt is an example test file
   that exercises these functions: They are defined as:
   
   int  example(char *s, int n);
   char examplechr(char c);
   char *examplestr(char *s);
   int  exampletf(int bool);

   to show a basic setup. Remove these functions and their prototypes
   when adapting a copy of custom.c to test your own functions.

   The basic if..then..else chain of function calls in calladapter()
   should be preserved, but any or all if-actions can be converted into code
   blocks and/or may call additional privately defined functions, which may
   be needed to convert the simple string and int parameters into something
   more elaborate. Similarly, you can add private functions that may be
   needed to set up more more complex test environments and trigger them
   from entries in the if..then..else chain in the same way as these
   example functions.

   The functions used to return results, report errors, etc, are defined
   in testharness.c as the setxxxxreply() functions.

   The final item in the condition chain should not be modified. It is there to
   trap function names in test scripts that don't match any of the functions
   being tested.

   When call adapter is called by testharness.c:

   - debug (normally zero) may be set to 1 to report calladapter entry and
           exit conditions.

   - fn    must contains the name of the function being tested

   - s[][] is an array containing the call parameters as strings

   - n[]   is an array containing the call parameters as int values with
           those that weren't numeric set to zero.

   It is the customiser's responsibility to map the contents of the
   s[][] and n[] arrays to the parameter list of each function being
   tested. Any additional code that may is needed to provide the correct
   mapping can be added as required.

   On exit calladapter() should return 0 if the overall call didn't
   encounter any fatal errors and a non-zero value if an error was
   encountered that was serious enough to abandon the run part-way
   through the test script.
*/
int calladapter(int debug, char *fn, char s[MAXARGS][MAXLINE], int n[MAXARGS])
{
   int   reply = 0;
   char  emsg[MAXLINE+1];

   if (debug > 1)
      teststart(fn, s, n);

   if (strcmp(fn, "example") == 0)
   {
      setintreply(example(s[0], n[1]));
   }
   else
   if (strcmp(fn, "examplechr") == 0)
   {
      setcharreply(examplechr(s[0][0]));
   }
   else
   if (strcmp(fn, "examplestr") == 0)
   {
      setstringreply(examplestr(s[0]));
   }
   else
   if (strcmp(fn, "exampletf") == 0)
   {
      setboolreply(exampletf(n[0]));
   }
   else
   if (strcmp(fn, "showdouble") == 0)
   {
      setdoublereply(3.141593);
   }
   else
   if (strcmp(fn, "showfloat") == 0)
   {
      setfloatreply(22.0/7.0);
   }
   else
   {
      snprintf(emsg, MAXLINE, "unknown function: %s", fn);
      seterror(emsg);
   }

   if (debug > 1)
      testend(reply, fn, s, n);

   return reply;
}
 
/*
   The text in showhelp() is displayed if the test harness is run
   with the '-?' option. It should say that it is a customised
   test harness, describe what functions or module are being tested
   and identify the (set of) test scripts it uses. 
*/
void showhelp()
{
   printf("\ncustom v%s %s\n\n",VERSION,COPYRIGHT);
   printf("Syntax  : custom [<opts>] <testfile...> [<opts>]\n");
   printf("Function: Example of the custom module.\n");
   printf("          The example test script is called data/custom01.txt\n");
   printf("Options :\n");
   printf("     -d           debug mode. Repeat for more detail.\n");
}

/*
   Private functions start here
   ============================
*/

/*
   example() is a dummy function, put here so that the prototype
   custom.c source file will compile, link and run the custom01.txt
   test script. It writes 's' to stdout and returns 'n'.
*/
int example(char *s, int n)
{
   printf("%s\n", s);
   return n;
}

/*
   examplechr() is a dummy function, put here so that the prototype
   custom.c source file will compile, link and run the custom01.txt
   test script. It returns 's';
*/
char examplechr(char c)
{
   return c;
}

/*
   examplestr() is a dummy function, put here so that the prototype
   custom.c source file will compile, link and run the custom01.txt
   test script. It returns 's';
*/
char *examplestr(char *s)
{
   return s;
}

/*
   exampletf() is a dummy function, put here so that the prototype
   custom.c source file will compile, link and run the custom01.txt
   test script. 'bool' is logical TRUE or FALSE value.
   It is returned as a boolean result.
*/
int exampletf(int bool)
{
   return bool;
}

/*
   testargs() is a debug aid that lists the supplied function parameters
*/
void testargs(char s[MAXARGS][MAXLINE], int n[MAXARGS])
{
   int i;

   fprintf(stderr, "    ");
   for (i = 0; i < MAXARGS && strlen(s[i]) > 0; i++)
      fprintf(stderr, "s[%d]='%s' ", i, s[i]);

   fprintf(stderr, "\n    ");
   for (i = 0; i < MAXARGS && strlen(s[i]) > 0; i++)
      fprintf(stderr, "n[%d]=%d ", i, n[i]);

   fprintf(stderr, "\n"); 
}

/*
   testend() is a debug aid called on exit from calladapter()
*/
void testend(int reply, char *fn, char s[MAXARGS][MAXLINE], int n[MAXARGS])
{
   fprintf(stderr, "%d = calladapter exit after %s\n", reply, fn);
   testargs(s, n);
}

/*
   teststart() is a debug aid called on entry to calladapter()
*/
void teststart(char *fn, char s[MAXARGS][MAXLINE], int n[MAXARGS])
{
   fprintf(stderr, "calladapter entered for %s\n", fn);
   testargs(s, n);
}
